@@ -1,16 +1,23 @@
+import json
+import logging
 import os
 import sys
+from typing import Any, Callable, Optional
 
 import click
+
 from dagster import check, seven
 from dagster.cli.workspace.cli_target import (
     get_working_directory_from_kwargs,
     python_origin_target_argument,
 )
-from dagster.core.events import EngineEventData
+from dagster.core.definitions.reconstruct import ReconstructablePipeline
+from dagster.core.errors import DagsterExecutionInterruptedError
+from dagster.core.events import DagsterEvent, DagsterEventType, EngineEventData
 from dagster.core.execution.api import create_execution_plan, execute_plan_iterator
 from dagster.core.execution.run_cancellation_thread import start_run_cancellation_thread
 from dagster.core.instance import DagsterInstance
+from dagster.core.origin import DEFAULT_DAGSTER_ENTRY_POINT, get_python_environment_entry_point
 from dagster.core.storage.pipeline_run import PipelineRun
 from dagster.core.test_utils import mock_system_timezone
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
@@ -20,12 +27,13 @@ from dagster.grpc.impl import core_execute_run
 from dagster.grpc.types import ExecuteRunArgs, ExecuteStepArgs, ResumeRunArgs
 from dagster.serdes import deserialize_as, serialize_dagster_namedtuple
 from dagster.seven import nullcontext
+from dagster.utils.error import serializable_error_info_from_exc_info
 from dagster.utils.hosted_user_process import recon_pipeline_from_origin
 from dagster.utils.interrupts import capture_interrupts
-from dagster.utils.log import default_system_logger
+from dagster.utils.log import configure_loggers
 
 
-@click.group(name="api")
+@click.group(name="api", hidden=True)
 def api_cli():
     """
     [INTERNAL] These commands are intended to support internal use cases. Users should generally
@@ -56,24 +64,37 @@ def execute_run_command(input_json):
             def send_to_buffer(event):
                 buffer.append(serialize_dagster_namedtuple(event))
 
-            _execute_run_command_body(
+            return_code = _execute_run_command_body(
                 recon_pipeline,
                 args.pipeline_run_id,
                 instance,
                 send_to_buffer,
+                set_exit_code_on_failure=args.set_exit_code_on_failure or False,
             )
 
             for line in buffer:
                 click.echo(line)
 
+            if return_code != 0:
+                sys.exit(return_code)
 
-def _execute_run_command_body(recon_pipeline, pipeline_run_id, instance, write_stream_fn):
+
+def _execute_run_command_body(
+    recon_pipeline: ReconstructablePipeline,
+    pipeline_run_id: str,
+    instance: DagsterInstance,
+    write_stream_fn: Callable[[DagsterEvent], Any],
+    set_exit_code_on_failure: bool,
+) -> int:
     if instance.should_start_background_run_thread:
         cancellation_thread, cancellation_thread_shutdown_event = start_run_cancellation_thread(
             instance, pipeline_run_id
         )
 
-    pipeline_run = instance.get_run_by_id(pipeline_run_id)
+    pipeline_run: PipelineRun = check.not_none(
+        instance.get_run_by_id(pipeline_run_id),
+        "Pipeline run with id '{}' not found for run execution.".format(pipeline_run_id),
+    )
 
     pid = os.getpid()
     instance.report_engine_event(
@@ -82,9 +103,20 @@ def _execute_run_command_body(recon_pipeline, pipeline_run_id, instance, write_s
         EngineEventData.in_process(pid, marker_end="cli_api_subprocess_init"),
     )
 
+    run_worker_failed = 0
+
     try:
-        for event in core_execute_run(recon_pipeline, pipeline_run, instance):
+        for event in core_execute_run(
+            recon_pipeline,
+            pipeline_run,
+            instance,
+        ):
             write_stream_fn(event)
+            if event.event_type == DagsterEventType.PIPELINE_FAILURE:
+                run_worker_failed = True
+    except:
+        # relies on core_execute_run writing failures to the event log before raising
+        run_worker_failed = True
     finally:
         if instance.should_start_background_run_thread:
             cancellation_thread_shutdown_event.set()
@@ -95,11 +127,14 @@ def _execute_run_command_body(recon_pipeline, pipeline_run_id, instance, write_s
                         "Cancellation thread did not shutdown gracefully",
                         pipeline_run,
                     )
+
         instance.report_engine_event(
             "Process for run exited (pid: {pid}).".format(pid=pid),
             pipeline_run,
         )
 
+    return 1 if (run_worker_failed and set_exit_code_on_failure) else 0
+
 
 @api_cli.command(
     name="resume_run",
@@ -124,23 +159,38 @@ def resume_run_command(input_json):
             def send_to_buffer(event):
                 buffer.append(serialize_dagster_namedtuple(event))
 
-            _resume_run_command_body(
+            return_code = _resume_run_command_body(
                 recon_pipeline,
                 args.pipeline_run_id,
                 instance,
                 send_to_buffer,
+                set_exit_code_on_failure=args.set_exit_code_on_failure or False,
             )
 
             for line in buffer:
                 click.echo(line)
 
+            if return_code != 0:
+                sys.exit(return_code)
 
-def _resume_run_command_body(recon_pipeline, pipeline_run_id, instance, write_stream_fn):
+
+def _resume_run_command_body(
+    recon_pipeline: ReconstructablePipeline,
+    pipeline_run_id: Optional[str],
+    instance: DagsterInstance,
+    write_stream_fn: Callable[[DagsterEvent], Any],
+    set_exit_code_on_failure: bool,
+):
     if instance.should_start_background_run_thread:
         cancellation_thread, cancellation_thread_shutdown_event = start_run_cancellation_thread(
             instance, pipeline_run_id
         )
     pipeline_run = instance.get_run_by_id(pipeline_run_id)
+    check.inst(
+        pipeline_run,
+        PipelineRun,
+        "Pipeline run with id '{}' not found for run execution.".format(pipeline_run_id),
+    )
 
     pid = os.getpid()
     instance.report_engine_event(
@@ -149,11 +199,22 @@ def _resume_run_command_body(recon_pipeline, pipeline_run_id, instance, write_st
         EngineEventData.in_process(pid, marker_end="cli_api_subprocess_init"),
     )
 
+    run_worker_failed = False
+
     try:
         for event in core_execute_run(
-            recon_pipeline, pipeline_run, instance, resume_from_failure=True
+            recon_pipeline,
+            pipeline_run,
+            instance,
+            resume_from_failure=True,
         ):
             write_stream_fn(event)
+            if event.event_type == DagsterEventType.PIPELINE_FAILURE:
+                run_worker_failed = True
+
+    except:
+        # relies on core_execute_run writing failures to the event log before raising
+        run_worker_failed = True
     finally:
         if instance.should_start_background_run_thread:
             cancellation_thread_shutdown_event.set()
@@ -169,6 +230,8 @@ def _resume_run_command_body(recon_pipeline, pipeline_run_id, instance, write_st
             pipeline_run,
         )
 
+    return 1 if (run_worker_failed and set_exit_code_on_failure) else 0
+
 
 def get_step_stats_by_key(instance, pipeline_run, step_keys_to_execute):
     # When using the k8s executor, there whould only ever be one step key
@@ -247,45 +310,13 @@ def execute_step_command(input_json):
             else DagsterInstance.get()
         ) as instance:
             pipeline_run = instance.get_run_by_id(args.pipeline_run_id)
-            check.inst(
-                pipeline_run,
-                PipelineRun,
-                "Pipeline run with id '{}' not found for step execution".format(
-                    args.pipeline_run_id
-                ),
-            )
-
-            if args.should_verify_step:
-                success = verify_step(
-                    instance,
-                    pipeline_run,
-                    args.known_state.get_retry_state(),
-                    args.step_keys_to_execute,
-                )
-                if not success:
-                    return
-
-            recon_pipeline = recon_pipeline_from_origin(
-                args.pipeline_origin
-            ).subset_for_execution_from_existing_pipeline(pipeline_run.solids_to_execute)
-
-            execution_plan = create_execution_plan(
-                recon_pipeline,
-                run_config=pipeline_run.run_config,
-                step_keys_to_execute=args.step_keys_to_execute,
-                mode=pipeline_run.mode,
-                known_state=args.known_state,
-            )
 
             buff = []
 
-            for event in execute_plan_iterator(
-                execution_plan,
-                recon_pipeline,
-                pipeline_run,
+            for event in _execute_step_command_body(
+                args,
                 instance,
-                run_config=pipeline_run.run_config,
-                retry_mode=args.retry_mode,
+                pipeline_run,
             ):
                 buff.append(serialize_dagster_namedtuple(event))
 
@@ -293,6 +324,68 @@ def execute_step_command(input_json):
                 click.echo(line)
 
 
+def _execute_step_command_body(
+    args: ExecuteStepArgs, instance: DagsterInstance, pipeline_run: PipelineRun
+):
+    single_step_key = (
+        args.step_keys_to_execute[0]
+        if args.step_keys_to_execute and len(args.step_keys_to_execute) == 1
+        else None
+    )
+    try:
+        check.inst(
+            pipeline_run,
+            PipelineRun,
+            "Pipeline run with id '{}' not found for step execution".format(args.pipeline_run_id),
+        )
+
+        if args.should_verify_step:
+            success = verify_step(
+                instance,
+                pipeline_run,
+                check.not_none(args.known_state).get_retry_state(),
+                args.step_keys_to_execute,
+            )
+            if not success:
+                return
+
+        recon_pipeline = recon_pipeline_from_origin(
+            args.pipeline_origin
+        ).subset_for_execution_from_existing_pipeline(pipeline_run.solids_to_execute)
+
+        execution_plan = create_execution_plan(
+            recon_pipeline,
+            run_config=pipeline_run.run_config,
+            step_keys_to_execute=args.step_keys_to_execute,
+            mode=pipeline_run.mode,
+            known_state=args.known_state,
+        )
+
+        yield from execute_plan_iterator(
+            execution_plan,
+            recon_pipeline,
+            pipeline_run,
+            instance,
+            run_config=pipeline_run.run_config,
+            retry_mode=args.retry_mode,
+        )
+    except (KeyboardInterrupt, DagsterExecutionInterruptedError):
+        yield instance.report_engine_event(
+            message="Step execution terminated by interrupt",
+            pipeline_run=pipeline_run,
+            step_key=single_step_key,
+        )
+        raise
+    except Exception:
+        yield instance.report_engine_event(
+            "An exception was thrown during step execution that is likely a framework error, rather than an error in user code.",
+            pipeline_run,
+            EngineEventData.engine_error(serializable_error_info_from_exc_info(sys.exc_info())),
+            step_key=single_step_key,
+        )
+        raise
+
+
 @api_cli.command(name="grpc", help="Serve the Dagster inter-process API over GRPC")
 @click.option(
     "--port",
@@ -349,6 +442,25 @@ def execute_step_command(input_json):
     "is managed directly from Dagit",
 )
 @python_origin_target_argument
+@click.option(
+    "--use-python-environment-entry-point",
+    is_flag=True,
+    required=False,
+    default=False,
+    help="If this flag is set, the server will signal to clients that they should launch "
+    "dagster commands using `<this server's python executable> -m dagster`, instead of the "
+    "default `dagster` entry point. This is useful when there are multiple Python environments "
+    "running in the same machine, so a single `dagster` entry point is not enough to uniquely "
+    "determine the environment.",
+)
+@click.option(
+    "--empty-working-directory",
+    is_flag=True,
+    required=False,
+    default=False,
+    help="Indicates that the working directory should be empty and should not set to the current "
+    "directory as a default",
+)
 @click.option(
     "--ipc-output-file",
     type=click.Path(),
@@ -377,6 +489,13 @@ def execute_step_command(input_json):
     default="INFO",
     help="Level at which to log output from the gRPC server process",
 )
+@click.option(
+    "--container-context",
+    type=click.STRING,
+    required=False,
+    help="Serialized JSON with configuration for any containers created to run the "
+    "code from this server.",
+)
 def grpc_command(
     port=None,
     socket=None,
@@ -389,6 +508,8 @@ def grpc_command(
     fixed_server_id=None,
     override_system_timezone=None,
     log_level="INFO",
+    use_python_environment_entry_point=False,
+    container_context=None,
     **kwargs,
 ):
     if seven.IS_WINDOWS and port is None:
@@ -398,7 +519,8 @@ def grpc_command(
     if not (port or socket and not (port and socket)):
         raise click.UsageError("You must pass one and only one of --port/-p or --socket/-s.")
 
-    logger = default_system_logger("dagster-code-server", coerce_valid_log_level(log_level))
+    configure_loggers(log_level=coerce_valid_log_level(log_level))
+    logger = logging.getLogger("dagster.code_server")
 
     loadable_target_origin = None
     if any(
@@ -415,7 +537,11 @@ def grpc_command(
         loadable_target_origin = LoadableTargetOrigin(
             executable_path=sys.executable,
             attribute=kwargs["attribute"],
-            working_directory=get_working_directory_from_kwargs(kwargs),
+            working_directory=(
+                None
+                if kwargs.get("empty_working_directory")
+                else get_working_directory_from_kwargs(kwargs)
+            ),
             module_name=kwargs["module_name"],
             python_file=kwargs["python_file"],
             package_name=kwargs["package_name"],
@@ -437,6 +563,12 @@ def grpc_command(
             lazy_load_user_code=lazy_load_user_code,
             ipc_output_file=ipc_output_file,
             fixed_server_id=fixed_server_id,
+            entry_point=(
+                get_python_environment_entry_point(sys.executable)
+                if use_python_environment_entry_point
+                else DEFAULT_DAGSTER_ENTRY_POINT
+            ),
+            container_context=json.loads(container_context) if container_context != None else None,
         )
 
         code_desc = " "
