@@ -6,9 +6,10 @@ from enum import Enum
 from typing import Any, Callable, Dict, Generic, List, NamedTuple, Optional, TypeVar, Union, cast
 
 import pendulum
+from dateutil.relativedelta import relativedelta
+
 from dagster import check
 from dagster.serdes import whitelist_for_serdes
-from dateutil.relativedelta import relativedelta
 
 from ...seven.compat.pendulum import PendulumDateTime, to_timezone
 from ...utils import frozenlist, merge_dicts
@@ -26,7 +27,11 @@ from ..storage.pipeline_run import PipelineRun
 from ..storage.tags import check_tags
 from .mode import DEFAULT_MODE_NAME
 from .run_request import RunRequest, SkipReason
-from .schedule_definition import ScheduleDefinition, ScheduleEvaluationContext
+from .schedule_definition import (
+    DefaultScheduleStatus,
+    ScheduleDefinition,
+    ScheduleEvaluationContext,
+)
 from .utils import check_valid_name
 
 DEFAULT_DATE_FORMAT = "%Y-%m-%d"
@@ -184,6 +189,12 @@ class StaticPartitionsDefinition(
 ):  # pylint: disable=unsubscriptable-object
     def __init__(self, partition_keys: List[str]):
         check.list_param(partition_keys, "partition_keys", of_type=str)
+
+        # Dagit selects partition ranges following the format '2022-01-13...2022-01-14'
+        # "..." is an invalid substring in partition keys
+        if any(["..." in partition_key for partition_key in partition_keys]):
+            raise DagsterInvalidDefinitionError("'...' is an invalid substring in a partition key")
+
         self._partitions = [Partition(key) for key in partition_keys]
 
     def get_partitions(
@@ -191,6 +202,9 @@ class StaticPartitionsDefinition(
     ) -> List[Partition[str]]:
         return self._partitions
 
+    def __hash__(self):
+        return hash(self.__repr__())
+
     def __eq__(self, other) -> bool:
         return (
             isinstance(other, StaticPartitionsDefinition)
@@ -292,21 +306,36 @@ class ScheduleTimeBasedPartitionsDefinition(
 
     def get_execution_time_to_partition_fn(self) -> Callable[[datetime], datetime]:
         if self.schedule_type is ScheduleType.HOURLY:
+            # Using subtract(minutes=d.minute) here instead of .replace(minute=0) because on
+            # pendulum 1, replace(minute=0) sometimes changes the timezone:
+            # >>> a = create_pendulum_time(2021, 11, 7, 0, 0, tz="US/Central")
+            #
+            # >>> a.add(hours=1)
+            # <Pendulum [2021-11-07T01:00:00-05:00]>
+            # >>> a.add(hours=1).replace(minute=0)
+            # <Pendulum [2021-11-07T01:00:00-06:00]>
             return lambda d: pendulum.instance(d).subtract(hours=self.offset, minutes=d.minute)
         elif self.schedule_type is ScheduleType.DAILY:
-            return lambda d: pendulum.instance(d).subtract(
-                days=self.offset, hours=d.hour, minutes=d.minute
+            return (
+                lambda d: pendulum.instance(d).replace(hour=0, minute=0).subtract(days=self.offset)
             )
         elif self.schedule_type is ScheduleType.WEEKLY:
             execution_day = cast(int, self.execution_day)
             day_difference = (execution_day - (self.start.weekday() + 1)) % 7
-            return lambda d: pendulum.instance(d).subtract(
-                weeks=self.offset, days=day_difference, hours=d.hour, minutes=d.minute
+            return (
+                lambda d: pendulum.instance(d)
+                .replace(hour=0, minute=0)
+                .subtract(
+                    weeks=self.offset,
+                    days=day_difference,
+                )
             )
         elif self.schedule_type is ScheduleType.MONTHLY:
             execution_day = cast(int, self.execution_day)
-            return lambda d: pendulum.instance(d).subtract(
-                months=self.offset, days=execution_day - 1, hours=d.hour, minutes=d.minute
+            return (
+                lambda d: pendulum.instance(d)
+                .replace(hour=0, minute=0)
+                .subtract(months=self.offset, days=execution_day - 1)
             )
         else:
             check.assert_never(self.schedule_type)
@@ -381,8 +410,8 @@ class PartitionSetDefinition(Generic[T]):
             "Only one of `partition_fn` or `partitions_def` must be supplied.",
         )
         check.invariant(
-            not (pipeline_name and job_name),
-            "Only one of `job_name` and `pipeline_name` must be supplied.",
+            (pipeline_name or job_name) and not (pipeline_name and job_name),
+            "Exactly one one of `job_name` and `pipeline_name` must be supplied.",
         )
 
         _wrap_partition_fn = None
@@ -449,6 +478,11 @@ class PartitionSetDefinition(Generic[T]):
     def job_name(self):
         return self._job_name
 
+    @property
+    def pipeline_or_job_name(self) -> str:
+        # one is guaranteed to be set
+        return cast(str, self._pipeline_name or self._job_name)
+
     @property
     def solid_selection(self):
         return self._solid_selection
@@ -483,7 +517,7 @@ class PartitionSetDefinition(Generic[T]):
             if partition.name == name:
                 return partition
 
-        check.failed("Partition name {} not found!".format(name))
+        raise DagsterUnknownPartitionError(f"Could not find a partition with key `{name}`")
 
     def get_partition_names(self, current_time: Optional[datetime] = None) -> List[str]:
         return [part.name for part in self.get_partitions(current_time)]
@@ -499,6 +533,7 @@ class PartitionSetDefinition(Generic[T]):
         description=None,
         decorated_fn=None,
         job=None,
+        default_status=DefaultScheduleStatus.STOPPED,
     ):
         """Create a ScheduleDefinition from a PartitionSetDefinition.
 
@@ -518,6 +553,8 @@ class PartitionSetDefinition(Generic[T]):
                 Supported strings for timezones are the ones provided by the
                 `IANA time zone database <https://www.iana.org/time-zones>` - e.g. "America/Los_Angeles".
             description (Optional[str]): A human-readable description of the schedule.
+            default_status (DefaultScheduleStatus): Whether the schedule starts as running or not. The default
+                status can be overridden from Dagit or via the GraphQL API.
 
         Returns:
             PartitionScheduleDefinition: The generated PartitionScheduleDefinition for the partition
@@ -531,6 +568,7 @@ class PartitionSetDefinition(Generic[T]):
         check.callable_param(partition_selector, "partition_selector")
         check.opt_str_param(execution_timezone, "execution_timezone")
         check.opt_str_param(description, "description")
+        check.inst_param(default_status, "default_status", DefaultScheduleStatus)
 
         def _execution_fn(context):
             check.inst_param(context, "context", ScheduleEvaluationContext)
@@ -556,10 +594,12 @@ class PartitionSetDefinition(Generic[T]):
                 yield SkipReason("Partition selector returned an empty list of partitions.")
                 return
 
+            partition_names = self.get_partition_names(context.scheduled_execution_time)
+
             missing_partition_names = [
                 partition.name
                 for partition in selected_partitions
-                if partition.name not in self.get_partition_names(context.scheduled_execution_time)
+                if partition.name not in partition_names
             ]
 
             if missing_partition_names:
@@ -615,6 +655,7 @@ class PartitionSetDefinition(Generic[T]):
             description=description,
             decorated_fn=decorated_fn,
             job=job,
+            default_status=default_status,
         )
 
 
@@ -638,6 +679,7 @@ class PartitionScheduleDefinition(ScheduleDefinition):
         description=None,
         decorated_fn=None,
         job=None,
+        default_status=DefaultScheduleStatus.STOPPED,
     ):
         super(PartitionScheduleDefinition, self).__init__(
             name=check_valid_name(name),
@@ -653,6 +695,7 @@ class PartitionScheduleDefinition(ScheduleDefinition):
             execution_fn=execution_fn,
             description=description,
             job=job,
+            default_status=default_status,
         )
         self._partition_set = check.inst_param(
             partition_set, "partition_set", PartitionSetDefinition
@@ -706,12 +749,16 @@ class PartitionedConfig(Generic[T]):
         partitions_def: PartitionsDefinition[T],  # pylint: disable=unsubscriptable-object
         run_config_for_partition_fn: Callable[[Partition[T]], Dict[str, Any]],
         decorated_fn: Optional[Callable[..., Dict[str, Any]]] = None,
+        tags_for_partition_fn: Optional[Callable[[Partition[T]], Dict[str, str]]] = None,
     ):
         self._partitions = check.inst_param(partitions_def, "partitions_def", PartitionsDefinition)
         self._run_config_for_partition_fn = check.callable_param(
             run_config_for_partition_fn, "run_config_for_partition_fn"
         )
         self._decorated_fn = decorated_fn
+        self._tags_for_partition_fn = check.opt_callable_param(
+            tags_for_partition_fn, "tags_for_partition_fn"
+        )
 
     @property
     def partitions_def(self) -> PartitionsDefinition[T]:  # pylint: disable=unsubscriptable-object
@@ -721,20 +768,24 @@ class PartitionedConfig(Generic[T]):
     def run_config_for_partition_fn(self) -> Callable[[Partition[T]], Dict[str, Any]]:
         return self._run_config_for_partition_fn
 
+    @property
+    def tags_for_partition_fn(self) -> Optional[Callable[[Partition[T]], Dict[str, str]]]:
+        return self._tags_for_partition_fn
+
     def get_partition_keys(self, current_time: Optional[datetime] = None) -> List[str]:
         return [partition.name for partition in self.partitions_def.get_partitions(current_time)]
 
-    def get_run_config(self, partition_key: str) -> Dict[str, Any]:
-        matching = [
-            partition
-            for partition in self.partitions_def.get_partitions()
-            if partition.name == partition_key
-        ]
-        if not matching:
-            raise DagsterUnknownPartitionError(
-                f"Could not find a partition with key `{partition_key}`"
-            )
-        return self.run_config_for_partition_fn(matching[0])
+    def get_run_config_for_partition_key(self, partition_key: str) -> Dict[str, Any]:
+        """Generates the run config corresponding to a partition key.
+
+        Args:
+            partition_key (str): the key for a partition that should be used to generate a run config.
+        """
+        partitions = self.partitions_def.get_partitions()
+        partition = [p for p in partitions if p.name == partition_key]
+        if len(partition) == 0:
+            raise DagsterInvalidInvocationError(f"No partition for partition key {partition_key}.")
+        return self.run_config_for_partition_fn(partition[0])
 
     def __call__(self, *args, **kwargs):
         if self._decorated_fn is None:
@@ -748,6 +799,7 @@ class PartitionedConfig(Generic[T]):
 
 def static_partitioned_config(
     partition_keys: List[str],
+    tags_for_partition_fn: Optional[Callable[[str], Dict[str, str]]] = None,
 ) -> Callable[[Callable[[str], Dict[str, Any]]], PartitionedConfig]:
     """Creates a static partitioned config for a job.
 
@@ -777,10 +829,14 @@ def static_partitioned_config(
         def _run_config_wrapper(partition: Partition[T]) -> Dict[str, Any]:
             return fn(partition.name)
 
+        def _tag_wrapper(partition: Partition[T]) -> Dict[str, str]:
+            return tags_for_partition_fn(partition.name) if tags_for_partition_fn else {}
+
         return PartitionedConfig(
             partitions_def=StaticPartitionsDefinition(partition_keys),
             run_config_for_partition_fn=_run_config_wrapper,
             decorated_fn=fn,
+            tags_for_partition_fn=_tag_wrapper,
         )
 
     return inner
@@ -788,6 +844,7 @@ def static_partitioned_config(
 
 def dynamic_partitioned_config(
     partition_fn: Callable[[Optional[datetime]], List[str]],
+    tags_for_partition_fn: Optional[Callable[[str], Dict[str, str]]] = None,
 ) -> Callable[[Callable[[str], Dict[str, Any]]], PartitionedConfig]:
     """Creates a dynamic partitioned config for a job.
 
@@ -812,10 +869,14 @@ def dynamic_partitioned_config(
         def _run_config_wrapper(partition: Partition[T]) -> Dict[str, Any]:
             return fn(partition.name)
 
+        def _tag_wrapper(partition: Partition[T]) -> Dict[str, str]:
+            return tags_for_partition_fn(partition.name) if tags_for_partition_fn else {}
+
         return PartitionedConfig(
             partitions_def=DynamicPartitionsDefinition(partition_fn),
             run_config_for_partition_fn=_run_config_wrapper,
             decorated_fn=fn,
+            tags_for_partition_fn=_tag_wrapper,
         )
 
     return inner
