@@ -1,7 +1,8 @@
-from collections import namedtuple
+from typing import Dict, List, NamedTuple, Optional
 
 import pkg_resources
 import yaml
+
 from dagster import check
 from dagster.core.definitions.utils import config_from_files, config_from_yaml_strings
 from dagster.core.errors import DagsterInvariantViolationError
@@ -12,7 +13,16 @@ from .utils import check_valid_name
 
 
 class PresetDefinition(
-    namedtuple("_PresetDefinition", "name run_config solid_selection mode tags")
+    NamedTuple(
+        "_PresetDefinition",
+        [
+            ("name", str),
+            ("run_config", Optional[Dict[str, object]]),
+            ("solid_selection", Optional[List[str]]),
+            ("mode", str),
+            ("tags", Dict[str, str]),
+        ],
+    )
 ):
     """Defines a preset configuration in which a pipeline can execute.
 
@@ -43,11 +53,11 @@ class PresetDefinition(
 
     def __new__(
         cls,
-        name,
-        run_config=None,
-        solid_selection=None,
-        mode=None,
-        tags=None,
+        name: str,
+        run_config: Optional[Dict[str, object]] = None,
+        solid_selection: Optional[List[str]] = None,
+        mode: Optional[str] = None,
+        tags: Optional[Dict[str, object]] = None,
     ):
 
         return super(PresetDefinition, cls).__new__(
@@ -69,7 +79,7 @@ class PresetDefinition(
             name (str): The name of this preset. Must be unique in the presets defined on a given
                 pipeline.
             config_files (Optional[List[str]]): List of paths or glob patterns for yaml files
-                to load and parse as the environment config for this preset.
+                to load and parse as the run config for this preset.
             solid_selection (Optional[List[str]]): A list of solid subselection (including single
                 solid names) to execute with the preset. e.g. ``['*some_solid+', 'other_solid']``
             mode (Optional[str]): The mode to apply when executing this preset. (default:
@@ -151,7 +161,7 @@ class PresetDefinition(
             name (str): The name of this preset. Must be unique in the presets defined on a given
                 pipeline.
             pkg_resource_defs (Optional[List[(str, str)]]): List of pkg_resource modules/files to
-                load as environment config for this preset.
+                load as the run config for this preset.
             solid_selection (Optional[List[str]]): A list of solid subselection (including single
                 solid names) to execute with this partition. e.g.
                 ``['*some_solid+', 'other_solid']``
@@ -199,10 +209,11 @@ class PresetDefinition(
         if run_config is None:
             return self
         else:
+            initial_config = self.run_config or {}
             return PresetDefinition(
                 name=self.name,
                 solid_selection=self.solid_selection,
                 mode=self.mode,
                 tags=self.tags,
-                run_config=deep_merge_dicts(self.run_config, run_config),
+                run_config=deep_merge_dicts(initial_config, run_config),
             )
