@@ -58,6 +58,7 @@ if TYPE_CHECKING:
         "LoadedInputData",
         "ComputeLogsCaptureData",
         "AssetObservationData",
+        "AssetMaterializationPlannedData",
     ]
 
 
@@ -75,6 +76,7 @@ class DagsterEventType(Enum):
     STEP_RESTARTED = "STEP_RESTARTED"
 
     ASSET_MATERIALIZATION = "ASSET_MATERIALIZATION"
+    ASSET_MATERIALIZATION_PLANNED = "ASSET_MATERIALIZATION_PLANNED"
     ASSET_OBSERVATION = "ASSET_OBSERVATION"
     STEP_EXPECTATION_RESULT = "STEP_EXPECTATION_RESULT"
 
@@ -193,6 +195,7 @@ PIPELINE_RUN_STATUS_TO_EVENT_TYPE = {v: k for k, v in EVENT_TYPE_TO_PIPELINE_RUN
 ASSET_EVENTS = {
     DagsterEventType.ASSET_MATERIALIZATION,
     DagsterEventType.ASSET_OBSERVATION,
+    DagsterEventType.ASSET_MATERIALIZATION_PLANNED,
 }
 
 
@@ -229,6 +232,10 @@ def _validate_event_specific_data(
         check.inst_param(event_specific_data, "event_specific_data", EngineEventData)
     elif event_type == DagsterEventType.HOOK_ERRORED:
         check.inst_param(event_specific_data, "event_specific_data", HookErroredData)
+    elif event_type == DagsterEventType.ASSET_MATERIALIZATION_PLANNED:
+        check.inst_param(
+            event_specific_data, "event_specific_data", AssetMaterializationPlannedData
+        )
 
     return event_specific_data
 
@@ -367,6 +374,22 @@ class DagsterEvent(
         log_resource_event(log_manager, event)
         return event
 
+    @staticmethod
+    def asset_materialization_planned(
+        pipeline_name: str,
+        asset_key: AssetKey,
+        log_manager: DagsterLogManager,
+    ) -> "DagsterEvent":
+        event = DagsterEvent(
+            event_type_value=DagsterEventType.ASSET_MATERIALIZATION_PLANNED.value,
+            pipeline_name=pipeline_name,
+            message=f"{pipeline_name} intends to materialize asset {asset_key.to_string()}",
+            event_specific_data=AssetMaterializationPlannedData(asset_key),
+        )
+        log_level = logging.DEBUG
+        log_manager.log_dagster_event(level=log_level, msg=event.message or "", dagster_event=event)
+        return event
+
     def __new__(
         cls,
         event_type_value: str,
@@ -507,12 +530,18 @@ class DagsterEvent(
     def is_asset_observation(self) -> bool:
         return self.event_type == DagsterEventType.ASSET_OBSERVATION
 
+    @property
+    def is_asset_materialization_planned(self) -> bool:
+        return self.event_type == DagsterEventType.ASSET_MATERIALIZATION_PLANNED
+
     @property
     def asset_key(self) -> Optional[AssetKey]:
         if self.event_type == DagsterEventType.ASSET_MATERIALIZATION:
             return self.step_materialization_data.materialization.asset_key
         elif self.event_type == DagsterEventType.ASSET_OBSERVATION:
             return self.asset_observation_data.asset_observation.asset_key
+        elif self.event_type == DagsterEventType.ASSET_MATERIALIZATION_PLANNED:
+            return self.asset_materialization_planned_data.asset_key
         else:
             return None
 
@@ -570,6 +599,15 @@ class DagsterEvent(
         _assert_type("asset_observation_data", DagsterEventType.ASSET_OBSERVATION, self.event_type)
         return cast(AssetObservationData, self.event_specific_data)
 
+    @property
+    def asset_materialization_planned_data(self) -> "AssetMaterializationPlannedData":
+        _assert_type(
+            "asset_materialization_planned",
+            DagsterEventType.ASSET_MATERIALIZATION_PLANNED,
+            self.event_type,
+        )
+        return cast(AssetMaterializationPlannedData, self.event_specific_data)
+
     @property
     def step_expectation_result_data(self) -> "StepExpectationResultData":
         _assert_type(
@@ -669,8 +707,7 @@ class DagsterEvent(
     def step_input_event(
         step_context: StepExecutionContext, step_input_data: "StepInputData"
     ) -> "DagsterEvent":
-        step_input = step_context.step.step_input_named(step_input_data.input_name)
-        input_def = step_input.source.get_input_def(step_context.pipeline_def)
+        input_def = step_context.solid_def.input_def_named(step_input_data.input_name)
 
         return DagsterEvent.from_step(
             event_type=DagsterEventType.STEP_INPUT,
@@ -1067,6 +1104,7 @@ class DagsterEvent(
         upstream_output_name: Optional[str] = None,
         upstream_step_key: Optional[str] = None,
         message_override: Optional[str] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
     ) -> "DagsterEvent":
 
         message = f'Loaded input "{input_name}" using input manager "{manager_key}"'
@@ -1081,6 +1119,7 @@ class DagsterEvent(
                 manager_key=manager_key,
                 upstream_output_name=upstream_output_name,
                 upstream_step_key=upstream_step_key,
+                metadata_entries=metadata_entries if metadata_entries else [],
             ),
             message=message_override or message,
         )
@@ -1244,6 +1283,16 @@ class StepMaterializationData(
         )
 
 
+@whitelist_for_serdes
+class AssetMaterializationPlannedData(
+    NamedTuple("_AssetMaterializationPlannedData", [("asset_key", AssetKey)])
+):
+    def __new__(cls, asset_key: AssetKey):
+        return super(AssetMaterializationPlannedData, cls).__new__(
+            cls, asset_key=check.inst_param(asset_key, "asset_key", AssetKey)
+        )
+
+
 @whitelist_for_serdes
 class StepExpectationResultData(
     NamedTuple(
@@ -1450,6 +1499,7 @@ class LoadedInputData(
             ("manager_key", str),
             ("upstream_output_name", Optional[str]),
             ("upstream_step_key", Optional[str]),
+            ("metadata_entries", Optional[List[MetadataEntry]]),
         ],
     )
 ):
@@ -1459,6 +1509,7 @@ class LoadedInputData(
         manager_key: str,
         upstream_output_name: Optional[str] = None,
         upstream_step_key: Optional[str] = None,
+        metadata_entries: Optional[List[MetadataEntry]] = None,
     ):
         return super(LoadedInputData, cls).__new__(
             cls,
@@ -1466,6 +1517,9 @@ class LoadedInputData(
             manager_key=check.str_param(manager_key, "manager_key"),
             upstream_output_name=check.opt_str_param(upstream_output_name, "upstream_output_name"),
             upstream_step_key=check.opt_str_param(upstream_step_key, "upstream_step_key"),
+            metadata_entries=check.opt_list_param(
+                metadata_entries, "metadata_entries", of_type=MetadataEntry
+            ),
         )
 
 
