@@ -1,7 +1,8 @@
-from typing import TYPE_CHECKING, Any, Dict, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Union, cast
 
 from dagster import check
-from dagster.core.definitions.events import AssetKey
+from dagster.core.definitions.events import AssetKey, AssetObservation
+from dagster.core.definitions.metadata import MetadataEntry, PartitionMetadataEntry
 from dagster.core.definitions.op_definition import OpDefinition
 from dagster.core.definitions.partition_key_range import PartitionKeyRange
 from dagster.core.definitions.solid_definition import SolidDefinition
@@ -13,6 +14,7 @@ from dagster.core.errors import DagsterInvariantViolationError
 
 if TYPE_CHECKING:
     from dagster.core.definitions.resource_definition import Resources
+    from dagster.core.events import DagsterEvent
     from dagster.core.execution.context.system import StepExecutionContext
     from dagster.core.log_manager import DagsterLogManager
     from dagster.core.types.dagster_type import DagsterType
@@ -86,6 +88,10 @@ class InputContext:
             self._resources_contain_cm = isinstance(self._resources, IContainsGenerator)
             self._cm_scope_entered = False
 
+        self._events: List["DagsterEvent"] = []
+        self._observations: List[AssetObservation] = []
+        self._metadata_entries: List[Union[MetadataEntry, PartitionMetadataEntry]] = []
+
     def __enter__(self):
         if self._resources_cm:
             self._cm_scope_entered = True
@@ -199,6 +205,9 @@ class InputContext:
 
     @property
     def asset_key(self) -> Optional[AssetKey]:
+        if not self._name:
+            return None
+
         matching_input_defs = [
             input_def
             for input_def in cast(SolidDefinition, self._solid_def).input_defs
@@ -283,10 +292,81 @@ class InputContext:
 
         partition_key_range = self.asset_partition_key_range
         return TimeWindow(
-            partitions_def.time_window_for_partition_key(partition_key_range.start).start,
-            partitions_def.time_window_for_partition_key(partition_key_range.end).end,
+            # mypy thinks partitions_def is <nothing> here because ????
+            partitions_def.time_window_for_partition_key(partition_key_range.start).start,  # type: ignore
+            partitions_def.time_window_for_partition_key(partition_key_range.end).end,  # type: ignore
         )
 
+    def consume_events(self) -> Iterator["DagsterEvent"]:
+        """Pops and yields all user-generated events that have been recorded from this context.
+
+        If consume_events has not yet been called, this will yield all logged events since the call to `handle_input`. If consume_events has been called, it will yield all events since the last time consume_events was called. Designed for internal use. Users should never need to invoke this method.
+        """
+
+        events = self._events
+        self._events = []
+        yield from events
+
+    def add_input_metadata(
+        self,
+        metadata: Dict[str, Any],
+        description: Optional[str] = None,
+    ) -> None:
+        """Accepts a dictionary of metadata. Metadata entries will appear on the LOADED_INPUT event.
+        If the input is an asset, metadata will be attached to an asset observation.
+
+        The asset observation will be yielded from the run and appear in the event log.
+        Only valid if the context has an asset key.
+        """
+        from dagster.core.definitions.metadata import normalize_metadata
+        from dagster.core.events import DagsterEvent
+
+        metadata = check.dict_param(metadata, "metadata", key_type=str)
+        self._metadata_entries.extend(normalize_metadata(metadata, []))
+        if self.asset_key:
+            check.opt_str_param(description, "description")
+
+            observation = AssetObservation(
+                asset_key=self.asset_key,
+                description=description,
+                partition=self.asset_partition_key if self.has_asset_partitions else None,
+                metadata=metadata,
+            )
+            self._observations.append(observation)
+            if self._step_context:
+                self._events.append(DagsterEvent.asset_observation(self._step_context, observation))
+
+    def get_observations(
+        self,
+    ) -> List[AssetObservation]:
+        """Retrieve the list of user-generated asset observations that were observed via the context.
+
+        User-generated events that were yielded will not appear in this list.
+
+        **Examples:**
+
+        .. code-block:: python
+
+            from dagster import IOManager, build_input_context, AssetObservation
+
+            class MyIOManager(IOManager):
+                def load_input(self, context, obj):
+                    ...
+
+            def test_load_input():
+                mgr = MyIOManager()
+                context = build_input_context()
+                mgr.load_input(context)
+                observations = context.get_observations()
+                ...
+        """
+        return self._observations
+
+    def consume_metadata_entries(self) -> List[Union[MetadataEntry, PartitionMetadataEntry]]:
+        result = self._metadata_entries
+        self._metadata_entries = []
+        return result
+
 
 def build_input_context(
     name: Optional[str] = None,
