@@ -1,22 +1,20 @@
 import os
 import pickle
+import shutil
 import subprocess
 import sys
 from typing import TYPE_CHECKING, Iterator, Optional, cast
 
 from dagster import Field, StringSource, check, resource
 from dagster.core.code_pointer import FileCodePointer, ModuleCodePointer
-from dagster.core.definitions.reconstructable import (
-    ReconstructablePipeline,
-    ReconstructableRepository,
-)
+from dagster.core.definitions.reconstruct import ReconstructablePipeline, ReconstructableRepository
 from dagster.core.definitions.step_launcher import StepLauncher, StepRunRef
 from dagster.core.errors import raise_execution_interrupts
-from dagster.core.events import DagsterEvent
+from dagster.core.events import DagsterEvent, DagsterEventType
 from dagster.core.execution.api import create_execution_plan
 from dagster.core.execution.context.system import StepExecutionContext
 from dagster.core.execution.context_creation_pipeline import PlanExecutionContextManager
-from dagster.core.execution.plan.execute_step import core_dagster_event_sequence_for_step
+from dagster.core.execution.plan.execute_plan import dagster_event_sequence_for_step
 from dagster.core.instance import DagsterInstance
 from dagster.core.storage.file_manager import LocalFileHandle, LocalFileManager
 from dagster.serdes import deserialize_value
@@ -55,6 +53,8 @@ class LocalExternalStepLauncher(StepLauncher):
         run_id = step_context.pipeline_run.run_id
 
         step_run_dir = os.path.join(self.scratch_dir, run_id, step_run_ref.step_key)
+        if os.path.exists(step_run_dir):
+            shutil.rmtree(step_run_dir)
         os.makedirs(step_run_dir)
 
         step_run_ref_file_path = os.path.join(step_run_dir, PICKLED_STEP_RUN_REF_FILE_NAME)
@@ -102,6 +102,44 @@ def _module_in_package_dir(file_path: str, package_dir: str) -> str:
     return ".".join(without_extension.split(os.sep))
 
 
+def _upstream_events_and_runs(step_context: StepExecutionContext):
+    "Grabs the minimal set of output events and runs to inform a remote instance of how to load each output."
+    step_inputs = step_context.step.step_inputs
+    upstream_output_handles = set().union(
+        *(step_input.source.step_output_handle_dependencies for step_input in step_inputs)
+    )
+    current_run = step_context.pipeline_run
+    events = []
+    runs = []
+    while True:
+        runs.append(current_run)
+        # note: this would cost N db calls where N = number of parent runs
+        step_output_records = step_context.instance.all_logs(
+            current_run.run_id, of_type=DagsterEventType.STEP_OUTPUT
+        )
+        # if the parent run has yielded an StepOutput event for the given step output,
+        # we find the source run id
+        for r in step_output_records:
+            output_handle = r.dagster_event.step_output_data.step_output_handle
+            # if this output matches one of the required step outputs, add it to the list of
+            # required events
+            if output_handle in upstream_output_handles:
+                events.append(r)
+                upstream_output_handles.remove(output_handle)
+
+        if current_run.parent_run_id is None:
+            if upstream_output_handles:
+                step_context.log.warn(
+                    f"Could not find outputs in the logs for output handles: {upstream_output_handles}"
+                )
+            break
+
+        # else, keep looking backwards
+        current_run = step_context.instance.get_run_by_id(current_run.parent_run_id)
+
+    return events, runs
+
+
 def step_context_to_step_run_ref(
     step_context: StepExecutionContext,
     prior_attempts_count: int,
@@ -144,23 +182,24 @@ def step_context_to_step_run_ref(
                     container_image=recon_pipeline.repository.container_image,
                     executable_path=recon_pipeline.repository.executable_path,
                     entry_point=recon_pipeline.repository.entry_point,
+                    container_context=recon_pipeline.repository.container_context,
                 ),
                 pipeline_name=recon_pipeline.pipeline_name,
                 solids_to_execute=recon_pipeline.solids_to_execute,
             )
 
-    parent_run_id = step_context.pipeline_run.parent_run_id
-    parent_run = step_context.instance.get_run_by_id(parent_run_id) if parent_run_id else None
+    upstream_output_events, run_group = _upstream_events_and_runs(step_context)
     return StepRunRef(
         run_config=step_context.run_config,
         pipeline_run=step_context.pipeline_run,
         run_id=step_context.pipeline_run.run_id,
         step_key=step_context.step.key,
         retry_mode=retry_mode,
-        recon_pipeline=recon_pipeline,
+        recon_pipeline=recon_pipeline,  # type: ignore
         prior_attempts_count=prior_attempts_count,
         known_state=step_context.execution_plan.known_state,
-        parent_run=parent_run,
+        run_group=run_group,
+        upstream_output_events=upstream_output_events,
     )
 
 
@@ -180,11 +219,14 @@ def external_instance_from_step_run_ref(
         DagsterInstance: A DagsterInstance that can be used to execute an external step.
     """
     instance = DagsterInstance.ephemeral()
-    # re-execution expects the parent run to be available on the instance, so add these
-    if step_run_ref.parent_run:
+    # re-execution expects the parent run(s) to be available on the instance, so add these
+    for run in step_run_ref.run_group:
         # remove the pipeline_snapshot_id, as this instance doesn't have any snapshots
-        instance.add_run(step_run_ref.pipeline_run._replace(pipeline_snapshot_id=None))
-        instance.add_run(step_run_ref.parent_run._replace(pipeline_snapshot_id=None))
+        instance.add_run(run._replace(pipeline_snapshot_id=None))
+    # the can_load() function on the step context currently depends on reading output events
+    # from the instance, so we make sure the remote instance has the relevant events
+    for entry in step_run_ref.upstream_output_events:
+        instance.handle_new_event(entry)
     if event_listener_fn:
         instance.add_event_listener(step_run_ref.run_id, event_listener_fn)
     return instance
@@ -195,7 +237,7 @@ def step_run_ref_to_step_context(
 ) -> StepExecutionContext:
     check.inst_param(instance, "instance", DagsterInstance)
     pipeline = step_run_ref.recon_pipeline.subset_for_execution_from_existing_pipeline(
-        step_run_ref.pipeline_run.solids_to_execute
+        frozenset(step_run_ref.pipeline_run.solids_to_execute or set())
     )
 
     execution_plan = create_execution_plan(
@@ -227,6 +269,7 @@ def step_run_ref_to_step_context(
     # Since we are launching from a PlanExecutionContext, the type will always be
     # StepExecutionContext.
     step_execution_context = cast(StepExecutionContext, step_execution_context)
+
     return step_execution_context
 
 
@@ -235,4 +278,7 @@ def run_step_from_ref(
 ) -> Iterator[DagsterEvent]:
     check.inst_param(instance, "instance", DagsterInstance)
     step_context = step_run_ref_to_step_context(step_run_ref, instance)
-    return core_dagster_event_sequence_for_step(step_context)
+
+    # The step should be forced to run locally with respect to the remote process that this step
+    # context is being deserialized in
+    return dagster_event_sequence_for_step(step_context, force_local_execution=True)
