@@ -1,11 +1,12 @@
 import inspect
 from collections import deque
+from contextlib import ContextDecorator
 from typing import AbstractSet, Any, Callable, Deque, Dict, Optional, cast
 
 from dagster import check
 from dagster.core.decorator_utils import get_function_params
-from dagster.core.definitions.pipeline import PipelineDefinition
-from dagster.core.definitions.resource import (
+from dagster.core.definitions.pipeline_definition import PipelineDefinition
+from dagster.core.definitions.resource_definition import (
     ResourceDefinition,
     ScopedResourcesBuilder,
     is_context_provided,
@@ -109,7 +110,7 @@ def _core_resource_initialization_event_generator(
     pipeline_def_for_backwards_compat: Optional[PipelineDefinition],
 ):
 
-    pipeline_name = None
+    pipeline_name = ""  # Must be initialized to a string to satisfy typechecker
     contains_generator = False
     if emit_persistent_events:
         check.invariant(
@@ -123,7 +124,7 @@ def _core_resource_initialization_event_generator(
     try:
         if emit_persistent_events and resource_keys_to_init:
             yield DagsterEvent.resource_init_start(
-                cast(str, pipeline_name),
+                pipeline_name,
                 cast(ExecutionPlan, execution_plan),
                 resource_log_manager,
                 resource_keys_to_init,
@@ -168,19 +169,25 @@ def _core_resource_initialization_event_generator(
 
         if emit_persistent_events and resource_keys_to_init:
             yield DagsterEvent.resource_init_success(
-                cast(str, pipeline_name),
+                pipeline_name,
                 cast(ExecutionPlan, execution_plan),
                 resource_log_manager,
                 resource_instances,
                 resource_init_times,
             )
+
+        delta_res_keys = resource_keys_to_init - set(resource_instances.keys())
+        check.invariant(
+            not delta_res_keys,
+            f"resources instances do not align with resource to init, difference: {delta_res_keys}",
+        )
         yield ScopedResourcesBuilder(resource_instances, contains_generator)
     except DagsterUserCodeExecutionError as dagster_user_error:
         # Can only end up in this state if we attempt to initialize a resource, so
         # resource_keys_to_init cannot be empty
         if emit_persistent_events:
             yield DagsterEvent.resource_init_failure(
-                cast(str, pipeline_name),
+                pipeline_name,
                 cast(ExecutionPlan, execution_plan),
                 resource_log_manager,
                 resource_keys_to_init,
@@ -282,7 +289,9 @@ def single_resource_event_generator(context, resource_name, resource_def):
         msg_fn = lambda: "Error executing resource_fn on ResourceDefinition {name}".format(
             name=resource_name
         )
-        with user_code_error_boundary(DagsterResourceFunctionError, msg_fn):
+        with user_code_error_boundary(
+            DagsterResourceFunctionError, msg_fn, log_manager=context.log
+        ):
             try:
                 with time_execution_scope() as timer_result:
                     resource_or_gen = (
@@ -290,11 +299,15 @@ def single_resource_event_generator(context, resource_name, resource_def):
                         if is_context_provided(get_function_params(resource_def.resource_fn))
                         else resource_def.resource_fn()
                     )
+
                     # Flag for whether resource is generator. This is used to ensure that teardown
                     # occurs when resources are initialized out of execution.
-                    is_gen = inspect.isgenerator(resource_or_gen)
-                    gen = ensure_gen(resource_or_gen)
-                    resource = next(gen)
+                    is_gen = inspect.isgenerator(resource_or_gen) or isinstance(
+                        resource_or_gen, ContextDecorator
+                    )
+
+                    resource_iter = _wrapped_resource_iterator(resource_or_gen)
+                    resource = next(resource_iter)
                 resource = InitializedResource(
                     resource, format_duration(timer_result.millis), is_gen
                 )
@@ -308,9 +321,9 @@ def single_resource_event_generator(context, resource_name, resource_def):
     except DagsterUserCodeExecutionError as dagster_user_error:
         raise dagster_user_error
 
-    with user_code_error_boundary(DagsterResourceFunctionError, msg_fn):
+    with user_code_error_boundary(DagsterResourceFunctionError, msg_fn, log_manager=context.log):
         try:
-            next(gen)
+            next(resource_iter)
         except StopIteration:
             pass
         else:
@@ -319,14 +332,9 @@ def single_resource_event_generator(context, resource_name, resource_def):
             )
 
 
-def get_required_resource_keys_to_init(
-    execution_plan, pipeline_def, resolved_run_config, intermediate_storage_def
-):
+def get_required_resource_keys_to_init(execution_plan, pipeline_def, resolved_run_config):
     resource_keys = set()
 
-    if intermediate_storage_def is not None:
-        resource_keys = resource_keys.union(intermediate_storage_def.required_resource_keys)
-
     for step_handle, step in execution_plan.step_dict.items():
         if step_handle not in execution_plan.step_handles_to_execute:
             continue
@@ -336,34 +344,37 @@ def get_required_resource_keys_to_init(
             resource_keys = resource_keys.union(hook_def.required_resource_keys)
 
         resource_keys = resource_keys.union(
-            get_required_resource_keys_for_step(
-                pipeline_def, step, execution_plan, resolved_run_config, intermediate_storage_def
-            )
+            get_required_resource_keys_for_step(pipeline_def, step, execution_plan)
         )
 
-    return frozenset(resource_keys)
+    resource_defs = pipeline_def.get_mode_definition(resolved_run_config.mode).resource_defs
+    return frozenset(get_transitive_required_resource_keys(resource_keys, resource_defs))
 
 
-def get_required_resource_keys_for_step(
-    pipeline_def, execution_step, execution_plan, resolved_run_config, intermediate_storage_def
-):
+def get_transitive_required_resource_keys(required_resource_keys, resource_defs):
+
+    resource_dependencies = resolve_resource_dependencies(resource_defs)
+
+    transitive_required_resource_keys = set()
+
+    for resource_key in required_resource_keys:
+        transitive_required_resource_keys = transitive_required_resource_keys.union(
+            set(get_dependencies(resource_key, resource_dependencies))
+        )
+
+    return transitive_required_resource_keys
+
+
+def get_required_resource_keys_for_step(pipeline_def, execution_step, execution_plan):
     resource_keys = set()
 
-    mode_definition = pipeline_def.get_mode_definition(resolved_run_config.mode)
-
-    resource_dependencies = resolve_resource_dependencies(mode_definition.resource_defs)
-
-    # add all the intermediate storage resource keys
-    if intermediate_storage_def is not None:
-        resource_keys = resource_keys.union(intermediate_storage_def.required_resource_keys)
-
     # add all the solid compute resource keys
     solid_def = pipeline_def.get_solid(execution_step.solid_handle).definition
     resource_keys = resource_keys.union(solid_def.required_resource_keys)
 
     # add input type, input loader, and input io manager resource keys
     for step_input in execution_step.step_inputs:
-        input_def = step_input.source.get_input_def(pipeline_def)
+        input_def = solid_def.input_def_named(step_input.name)
 
         resource_keys = resource_keys.union(input_def.dagster_type.required_resource_keys)
 
@@ -397,15 +408,27 @@ def get_required_resource_keys_for_step(
         if output_def.io_manager_key:
             resource_keys = resource_keys.union([output_def.io_manager_key])
 
-    # add all the storage-compatible plugin resource keys
-    for dagster_type in solid_def.all_dagster_types():
-        for auto_plugin in dagster_type.auto_plugins:
-            if intermediate_storage_def is not None:
-                if auto_plugin.compatible_with_storage_def(intermediate_storage_def):
-                    resource_keys = resource_keys.union(auto_plugin.required_resource_keys())
-
-    for resource_name in resource_keys:
-        resource_keys = resource_keys.union(
-            set(get_dependencies(resource_name, resource_dependencies))
-        )
     return frozenset(resource_keys)
+
+
+def _wrapped_resource_iterator(resource_or_gen):
+    """Returns an iterator which yields a single item, which is the resource.
+
+    If the resource is not a context manager, then resource teardown happens following the first yield.
+    If the resource is a context manager, then resource initialization happens as the passed-in
+    context manager opens. Resource teardown happens as the passed-in context manager closes (which will occur after all compute is finished).
+    """
+
+    # Context managers created using contextlib.contextdecorator are not usable as iterators.
+    # Opening context manager and directly yielding preserves initialization/teardown behavior,
+    # while also letting the context manager be used as an iterator.
+    if isinstance(resource_or_gen, ContextDecorator):
+
+        def _gen_resource():
+            with resource_or_gen as resource:
+                yield resource
+
+        return _gen_resource()
+
+    # Otherwise, coerce to generator without opening context manager
+    return ensure_gen(resource_or_gen)
